import React, { useEffect, useState } from 'react'
import { Form, Input, InputNumber, Radio, Modal, Select } from 'antd';
import departmentService from '@/request/service/deparatment';
import userService from '@/request/service/user';
import { useRequest } from '@/hooks/use-request';
import type { DepartmentVO, UserDTO, UserVO } from '@/models'
import { antdUtils } from '@/utils/antd';

const layout = {
    labelCol: { span: 4, },
    wrapperCol: { span: 16 },
};

interface EditorProps {
    visible: boolean;
    onCancel: (flag?: boolean) => void;
    onSave: (role: DepartmentVO) => void;
    data?: DepartmentVO | null;
}


const DepartmentEditor: React.FC<EditorProps> = (props) => {

    const { visible, onCancel, onSave, data } = props;
    const [ departments, setDepartments ] = useState<DepartmentVO[]>([]);
    const [ userList, setUserList ] = useState<UserVO[]>([]);

    const { runAsync: updatePostApi } = useRequest(departmentService.updateDeptApi, { manual: true });
    const { runAsync: createPostApi } = useRequest(departmentService.createDeptApi, { manual: true });
    const { runAsync: listSimpleDeptApi } = useRequest(departmentService.listSimpleDeptApi, {manual: true});
    const { runAsync: listSimpleUsersApi } = useRequest(userService.getListSimpleUsersApi, {manual: true});

    const isEdit = !!data;

    const [saveLoading, setSaveLoading] = useState(false);
    const [form] = Form.useForm();

    const loadDepartments = async () => {
        const [error, { data, msg }] = await listSimpleDeptApi();
        if(!error) {
            setDepartments(data);
        } else {
            antdUtils.message?.error(msg ?? '获取部门列表数据失败');
        }
    }

    const loadUserList = async () => {
        const [error, { data, msg }] = await listSimpleUsersApi();
        if(!error) {
            setUserList(data);
        } else {
            antdUtils.message?.error(msg ?? '获取用户列表数据失败');
        }
    }

    useEffect(() => {
        if (visible) {
            if(departments.length === 0) {
                loadDepartments()
            }
            if(userList.length === 0) {
                loadUserList();
            }
            if (data) {
                form.setFieldsValue(data);
            }
        } else {
            form.resetFields();
        }
    }, [visible]);

    const save = async () => {
        await form.validateFields();
        setSaveLoading(true);
        const fieldValues = form.getFieldsValue();
        const newValue = isEdit ? { ...data, ...fieldValues } : fieldValues;
        const [error, { msg, code }] = isEdit ? await updatePostApi(newValue) : await createPostApi(newValue);
        if (!error && code === 0) {
            onSave(newValue);
        } else {
            antdUtils.message?.open({
                type: 'error',
                content: msg ?? '操作失败',
            });
        }
        setSaveLoading(false);
    }

    return (
        <>
            <Modal
                open={visible}
                title={isEdit ? "编辑" : "新建"}
                width={640}
                onOk={save}
                onCancel={() => onCancel()}
                confirmLoading={saveLoading}
                destroyOnClose
            >
                <Form
                    form={form}
                    {...layout}
                    onFinish={save}
                    labelCol={{ flex: '0 0 100px' }}
                    wrapperCol={{ span: 16 }}
                >
                    <Form.Item name="parentId" label="上级部门">
                        <Select options={departments.map(it => ({ value: it.id, label: it.name }))} />
                    </Form.Item>
                    <Form.Item name="name" label="部门名称"
                        rules={[
                            {
                                required: true,
                                message: '请输入岗位名称',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>

                    <Form.Item name="leaderUserId" label="负责人">
                        <Select  options={userList.map(it => ({ value: it.id, label: it.nickname }))}/>
                    </Form.Item>

                    <Form.Item name="phone" label="联系电话"
                        rules={[
                            {
                                required: true,
                                message: '请输入电话号码',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>
                    <Form.Item name="email" label="邮箱"
                        rules={[
                            {
                                required: true,
                                message: '请输入邮箱',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>

                    <Form.Item name="sort" label="显示排序"
                        rules={[
                            {
                                required: true,
                                message: '请输入显示排序',
                            },
                        ]}
                    >
                        <InputNumber min={1} max={100} />
                    </Form.Item>

                    <Form.Item name="status" label="状态"
                        rules={[
                            {
                                required: true,
                                message: '请设置状态',
                            },
                        ]}>
                        <Radio.Group options={[
                            { value: 0, label: "开启" },
                            { value: 1, label: "关闭" }
                        ]} optionType="default">
                        </Radio.Group>
                    </Form.Item>
                </Form>
            </Modal>
        </>
    )
}

export default DepartmentEditor;
