import React, { useState, useEffect } from 'react';
import { Space, Table, Button, Input, Select, Divider, Tag, Card, Badge, Form, DatePicker } from 'antd';
import type { TableColumnsType } from 'antd';
import type { TableProps } from 'antd/es/table';
import { t } from '@/utils/i18n';
import { PlusOutlined, ExclamationCircleFilled, SearchOutlined, UndoOutlined } from '@ant-design/icons';
import { antdUtils } from '@/utils/antd';
import { useRequest } from '@/hooks/use-request';
import { formatDate } from '@/utils/formatTime'
import roleService from '@/request/service/role';
import { RoleVO } from '@/models';
import RoleEditor from './role-editor';

const { RangePicker } = DatePicker;

export default () => {

    const [editorVisable, seEditorVisable] = useState<boolean>(false);
    const [editRole, setEditRole] = useState<RoleVO>();
    const [dataSource, setDataSource] = useState<RoleVO[]>([]);
    const [total, setTotal] = useState(0);
    const [page, setPage] = useState({
        pageNo: 1,
        pageSize: 10
    });
    const [searchFrom] = Form.useForm();

    const { runAsync: getRoleList } = useRequest(roleService.getRoleList, { manual: true });
    const { runAsync: deleteRole } = useRequest(roleService.deleteRole, { manual: true });

    const loadRoles = async () => {
        const params = {...page, ...searchFrom.getFieldsValue()}
        const createTime = searchFrom.getFieldValue("createTime");
        if(createTime) {
            params["createTime"] = `[${createTime[0].format('YYYY-MM-DD 00:00:00')},${createTime[1].format('YYYY-MM-DD 23:59:59')}]`;
        }
        const [error, { code, msg, data }] = await getRoleList(params);
        if (error || code !== 0) {
            antdUtils.message?.open({ type: 'error', content: msg ?? '操作失败' });
            return
        }
        setDataSource(data.list);
        setTotal(data.total);
    };

    const onReset = () => {
        searchFrom.resetFields();
        loadRoles();
    };

    const showDeleteConfirm = (role: RoleVO) => {
        antdUtils.modal?.confirm({
            title: '确认要将该角色删除吗？',
            icon: <ExclamationCircleFilled />,
            content: '请注意删除以后不可恢复！',
            okText: '删除',
            okType: 'danger',
            cancelText: '取消',
            onOk() {
                return new Promise(async (resolve) => {
                    const [error, { code, msg }] = await deleteRole(role.id);
                    if (error || code !== 0) {
                        antdUtils.message?.open({ type: 'error', content: msg ?? '操作失败' })
                    } else {
                        antdUtils.message?.open({ type: 'success', content: '删除成功' })
                    }
                    await loadRoles();
                    resolve('')
                }).catch(() => antdUtils.message?.open({
                    type: 'error',
                    content: '操作失败',
                }));
            },
            onCancel() {
            },
        });
    };

    const columns: TableColumnsType<RoleVO> = [
        {
            title: '角色编号',
            dataIndex: 'id',
            key: 'id',
            align: 'center',
            width: 100,
        },
        {
            title: '角色名称',
            dataIndex: 'name',
            key: 'name',
            align: 'center',
            width: 200
        },
        {
            title: '角色类型',
            dataIndex: 'type',
            key: 'type',
            align: 'center',
            width: 150,
            render: (value: number) => {
                return (value === 1 ? <Tag color="purple">内置</Tag> : <Tag color="blue">自定义</Tag>)
            }
        },
        {
            title: '角色标识',
            dataIndex: 'code',
            key: 'code',
            align: 'center',
            width: 150
        },
        {
            title: '显示顺序',
            key: 'sort',
            dataIndex: 'sort',
            align: 'center',
        },
        {
            title: '备注',
            key: 'remark',
            dataIndex: 'remark',
            align: 'center',
        },
        {
            title: '状态',
            key: 'status',
            dataIndex: 'status',
            width: 100,
            align: 'center',
            render: (value: number) => {
                return (value === 0 ? <Badge status="success" text="已开启" /> : <Badge status="error" text="已关闭" />)
            }
        },
        {
            title: '创建时间',
            key: 'createTime',
            dataIndex: 'createTime',
            width: 200,
            align: 'center',
            render: (value: number) => {
                return formatDate(new Date(value), "YYYY-mm-dd HH:MM:SS")
            }
        },
        {
            title: t("QkOmYwne" /* 操作 */),
            key: 'action',
            fixed: 'right',
            align: 'center',
            render: (role: RoleVO, record) => (
                <Space size="small" split={(<Divider type='vertical' />)}>
                    <a onClick={() => {
                        setEditRole(role);
                        seEditorVisable(true);
                    }}>
                        编辑
                    </a>
                    <a onClick={() => {

                    }}>
                        菜单权限
                    </a>
                    <a onClick={() => {

                    }}>
                        数据权限
                    </a>
                    <a onClick={() => {
                        showDeleteConfirm(role)
                    }}>
                        删除
                    </a>
                </Space>
            ),
        },
    ];

    const onChange: TableProps<RoleVO>['onChange'] = (pagination, filters, sorter, extra) => {
        setPage({
            pageNo: pagination.current??1,
            pageSize: pagination.pageSize??10
        });
    };

    useEffect(() => {
        loadRoles();
    }, [page]);

    return (
        <>
            <div style={{ minWidth: 1600 }}>
                <Card className='mt-[4px] dark:bg-[rgb(33,41,70)] bg-white roundle-lg px[12px]' bodyStyle={{ paddingTop: 4, paddingBottom: 4 }}>
                    <div className='flex justify-between content-center'>
                        <div className='flex justify-normal items-center'>
                            <Form layout='inline' form={searchFrom}>
                                <Form.Item name="name" label="角色名称">
                                    <Input className='w-[150px]' placeholder='请输入名称' allowClear />
                                </Form.Item>
                                <Form.Item name="code" label="角色标识">
                                    <Input className='w-[150px]' placeholder='请输入标识' allowClear />
                                </Form.Item>
                                <Form.Item className='ml-2 w-[130px]' name="status" label="状态">
                                    <Select placeholder="请选择" allowClear >
                                        <Select.Option value="">全部</Select.Option>
                                        <Select.Option value="0">开启</Select.Option>
                                        <Select.Option value="1">关闭</Select.Option>
                                    </Select>
                                </Form.Item>
                                <Form.Item className='ml-2 w-[380px]' name="createTime" label="创建时间">
                                    <RangePicker className='w-[260px]'/>
                                </Form.Item>
                            </Form>
                            <Space.Compact className="ml-5">
                                <Button type='primary' size='large' icon={<SearchOutlined />} onClick={loadRoles}> 搜索 </Button>
                                <Button type='primary' size='large' icon={<UndoOutlined />} onClick={onReset}> 重置 </Button>
                            </Space.Compact>
                        </div>
                        <div className="py-[4px] flex justify-normal items-center">
                            <Button className="ml-5" type='primary' size='large' icon={<PlusOutlined />} onClick={() => {
                                setEditRole(undefined);
                                seEditorVisable(true);
                            }}> 创建角色 </Button>
                        </div>
                    </div>
                </Card>
                <Card className='mt-[4px] dark:bg-[rgb(33,41,70)] bg-white roundle-lg px[12px]'
                    style={{height: '100%'}}
                    bodyStyle={{height: '100%'}}>
                    <Table rowKey="id"
                        scroll={{ x: true }}
                        style={{height: '100%'}}
                        columns={columns}
                        dataSource={dataSource}
                        className='bg-transparent'
                        onChange={onChange}
                        pagination={{ 
                            current: page.pageNo,
                            pageSize: page.pageSize,
                            total,
                            position: ['bottomRight'] 
                        }} />
                </Card>
            </div>
            <RoleEditor
                onSave={() => {
                    loadRoles();
                    seEditorVisable(false);
                }}
                onCancel={() => { seEditorVisable(false) }}
                visible={editorVisable}
                data={editRole} />
        </>
    );
};
